import * as React from 'react';
import { Feature } from '@common-stack/client-react';
import { Files,  /** Editors */ } from '@files-stack/client-react';
import { reducers, setRootPath, filesModule } from '@files-stack/client-state';
import {
    filesResolvers, defaults, editorResolvers, filesTreeResolvers, dataIdFromObject,
    modelContentChangeEpic, modelSaveEpic, fragmentMatcher,
    schema,
} from '@files-stack/client-state';
import { merge } from 'lodash';
import { IClientFileService, ClientTypes, IFilesTreeService, IUntitledEditorService,
     IClientService, IClientTextFileService, ITextFileEditorModel,
     } from '@files-stack/core';
import { DockLayout } from '@ide-layout/client-react';
import { interfaces } from 'inversify';
import { IEditorInput } from '@workbench-stack/core';
import { Editor } from '@workbench-stack/module-browser';

const actionFunctions = [setRootPath];

const filesServiceGen = (container: interfaces.Container): IClientService => {
    return {
        fileService: container.get<IClientFileService>(ClientTypes.IFileService),
        textFileService: container.get<IClientTextFileService>(ClientTypes.ITextFileService),
        filesTreeService: container.get<IFilesTreeService>(ClientTypes.IFilesTreeService),
        untitledEditorService: container.get<IUntitledEditorService>(ClientTypes.IUntitledEditorService),
        textFileEditorModel: container.get<ITextFileEditorModel>(ClientTypes.ITextFileEditorModel),
    };
};

// const Editor = ({ input }) => React.useMemo(() => EditorComponent({ input }), [input]);

const Content = ({ input }: { input: IEditorInput }) => {
    console.log('--Content Component should only render once per input', input);
    let props = {} as any;
    return  <Editor {...props} input={input} uri={input.resource} enableApi={true} />;
  };
  
export default new Feature({
    leftMainPanelItems: { files: <Files /> },
    middleMainPanelItems: { editors: <DockLayout contentComponent={Content} /> }, // <Editor/>
    middleMainPanelItemsProps: { editors: { actionFunctions } },
    reducer: { ...reducers },
    createContainerFunc: filesModule,
    clientStateParams: [{ resolvers: [filesResolvers, editorResolvers, filesTreeResolvers], defaults, 
        typeDefs: schema, fragmentMatcher: fragmentMatcher as any }],
    dataIdFromObject,
    createServiceFunc: filesServiceGen,
    epic: [modelContentChangeEpic, modelSaveEpic],
});
